<?php
namespace Ak\Hydrator\Strategy;

use Laminas\Hydrator\HydratorAwareInterface;
use Laminas\Hydrator\HydratorAwareTrait;
use Laminas\Hydrator\HydratorInterface;
use Laminas\Hydrator\ReflectionHydrator;
use Laminas\Hydrator\Strategy\StrategyInterface;

/**
 * Hidrira vrijednosti kao objekte.
 *
 * @package Ak\Hydrator\Strategy
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/hydrator
 */
final class ObjectStrategy implements
    HydratorAwareInterface,
    StrategyInterface
{
    use HydratorAwareTrait;

    /** @var string */
    private $objectClass;

    /**
     * Konstruktor.
     *
     * @param string $objectClass          Klasa objekta kojeg hidriramo.
     * @param HydratorInterface $hydrator  Hidrator korišten za hidriranje objekta date klase.
     */
    public function __construct(string $objectClass, HydratorInterface $hydrator = null)
    {
        $this->objectClass = $objectClass;
        $this->hydrator = $hydrator;
    }

    /**
     * Retrieve hydrator
     */
    public function getHydrator(): ?HydratorInterface
    {
        if (null === $this->hydrator) {
            $this->setHydrator(new ReflectionHydrator());
        }

        return $this->hydrator;
    }

    /**
     * Pretvara datu vrijednost za ekstrakciju.
     *
     * @param mixed $value
     * @param object|null $object
     *
     * @return mixed
     */
    public function extract($value, ?object $object = null)
    {
        $hydrator = $this->getHydrator();

        return $hydrator->extract($value);
    }

    /**
     * Pretvara datu vrijednost za hidraciju.
     *
     * @param mixed $value
     * @param array|null $data
     *
     * @return mixed
     */
    public function hydrate($value, ?array $data)
    {
        $hydrator = $this->getHydrator();
        $objectClass = $this->objectClass;

        return $hydrator->hydrate($value, new $objectClass());
    }
}
