<?php
namespace AkTest\Hydrator\Strategy;

use Ak\Hydrator\Strategy\ObjectStrategy;
use AkTest\Hydrator\Mock\ReadOnlyObject;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;
use stdClass;
use Laminas\Hydrator\HydratorInterface;

/**
 * Class HydratorStrategyTest
 *
 * @package AkTest\Hydrator\Strategy
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/wp-core
 */
final class ObjectStrategyTest extends TestCase
{
    public function testHydratorIsCalledForExtraction()
    {
        $value = (object) ['prop' => 'some-object'];

        /** @var HydratorInterface|MockObject $hydrator */
        $hydrator = $this->createMock(HydratorInterface::class);
        $hydrator->expects($this->once())
            ->method('extract')
            ->with($value);

        $strategy = new ObjectStrategy(stdClass::class, $hydrator);
        $strategy->extract($value);
    }

    public function testHydratorIsCalledForHydration()
    {
        $value = [
            'property' => 'some-value'
        ];

        /** @var HydratorInterface|MockObject $hydrator */
        $hydrator = $this->createMock(HydratorInterface::class);
        $hydrator->expects($this->once())
            ->method('hydrate')
            ->with($value, $this->isInstanceOf(stdClass::class));

        $strategy = new ObjectStrategy(stdClass::class, $hydrator);
        $strategy->hydrate($value, null);
    }

    public function testHasDefaultHydrator()
    {
        $value = ['property' => 'some-value'];
        $strategy = new ObjectStrategy(ReadOnlyObject::class);

        /** @var ReadOnlyObject $object */
        $object = $strategy->hydrate($value, null);

        $this->assertInstanceOf(ReadOnlyObject::class, $object);
        $this->assertEquals('some-value', $object->getProperty());
    }
}
